% Generate load values
load_min.ENG = 0.1;
steps = 101;
load_values = linspace(load_min.ENG, 1, steps);
% Minimum load engine

a_heat.MGO = 0.3; % from component sizing model data
a_heat.LNG = 0.36; % from component sizing model data

% Calculate y values
eta_generator = 0.95;
eta_LNG_values = eta_generator * 0.4032 * load_values.^0.1096;          % from wartsila 34DF (Ali), including generator efficiency
eta_MGO_values = eta_generator * (ENG.eta_el_MGO/ENG.eta_el_LNG) * 0.4032 * load_values.^0.1096;
% Calculate heat values
eta_heat_MGO_values = a_heat.MGO * eta_MGO_values;
eta_heat_LNG_values = ENG.eta_el_LNG * eta_LNG_values; % from wartsila 34DF (Ali)

% Create a MATLAB structure
ENG_map = struct('load', load_values, 'eta_MGO', eta_MGO_values, 'eta_LNG', eta_LNG_values,'eta_heat_MGO', eta_heat_MGO_values, 'eta_heat_LNG', eta_heat_LNG_values);

% Save the structure to a file
save('EngineData/Maps/ENG_map.mat', 'ENG_map');

% Efficiency-load curve
f = figure('position',[50,50, 800, 600]);
hold on, grid on, box on,
plot((load_values),eta_MGO_values, 'LineWidth', 1.2, 'Color', color.blue)
plot((load_values),eta_LNG_values, 'LineWidth', 1.2, 'Color', color.blue,'LineStyle','--')
plot((load_values),eta_heat_MGO_values, 'LineWidth', 1.2, 'Color', color.red)
plot((load_values),eta_heat_LNG_values, 'LineWidth', 1.2, 'Color', color.red,'LineStyle','--')
xlabel('Load [-]')
ylabel('Efficiency [-]')
ax = gca;
ax.FontSize = Fontsize;
axis([0 1 0 0.6])
legend('Net electric efficiency Fuel-mode','Net electric efficiency Gas-mode','Heat efficiency Fuel-mode','Heat efficiency Gas-mode','Location','northwest')
saveas(gcf,'EngineData/Maps/ENG_efficiency.png')

%% Emissions-load curve
%CO2
f = figure('position',[50,50, 800, 600],'visible','off');
hold on, grid on, box on,
plot(ENG_map_emissions.MGO.Load,ENG_map_emissions.MGO.s_CO2_MGO, 'LineWidth', 1.2, 'Color', color.grey)
plot(ENG_map_emissions.LNG.Load,ENG_map_emissions.LNG.s_CO2_LNG, 'LineWidth', 1.2, 'Color', color.grey,'LineStyle','--')
xlabel('Load [-]')
ylabel('Specific CO2 emissions [g/kWhe]')
ax = gca;
ax.FontSize = Fontsize;
axis([0 1 0 800])
legend('MGO','LNG','Location','northeast')
saveas(gcf,'EngineData/Maps/ENG_CO2.png')

f = figure('position',[50,50, 800, 600],'visible','off');
hold on, grid on, box on,
plot(ENG_map_emissions.MGO.Load,ENG_map_emissions.MGO.s_NOx_MGO, 'LineWidth', 1.2, 'Color', color.blue)
plot(ENG_map_emissions.LNG.Load,ENG_map_emissions.LNG.s_NOx_LNG, 'LineWidth', 1.2, 'Color', color.blue, 'LineStyle','--')
plot(ENG_map_emissions.MGO.Load,ENG_map_emissions.MGO.s_CH4_MGO, 'LineWidth', 1.2, 'Color', color.black)
plot(ENG_map_emissions.LNG.Load,ENG_map_emissions.LNG.s_CH4_LNG, 'LineWidth', 1.2, 'Color', color.black, 'LineStyle','--')
plot(ENG_map_emissions.LNG.Load,ENG_map_emissions.LNG.s_CO_LNG, 'LineWidth', 1.2, 'Color', color.yellow, 'LineStyle','--')
xlabel('Load [-]')
ylabel('Specific emissions [g/kWhe]')
ax = gca;
ax.FontSize = Fontsize;
axis([0 1 0 15])
legend('NOx - MGO', 'NOx - LNG', 'CH4 - MGO', 'CH4 - LNG', 'CO - LNG', 'Location','northeast')
saveas(gcf,'EngineData/Maps/ENG_NOx_CH4_CO.png')

f = figure('position',[50,50, 800, 600],'visible','off');
hold on, grid on, box on,
plot(ENG_map_emissions.MGO.Load,ENG_map_emissions.MGO.s_SOx_MGO, 'LineWidth', 1.2, 'Color', color.red)
plot(ENG_map_emissions.LNG.Load,ENG_map_emissions.LNG.s_SOx_LNG, 'LineWidth', 1.2, 'Color', color.red, 'LineStyle','--')
plot(ENG_map_emissions.MGO.Load,ENG_map_emissions.MGO.s_PM_MGO, 'LineWidth', 1.2, 'Color', color.green)
plot(ENG_map_emissions.LNG2.Load,ENG_map_emissions.LNG2.s_PM_LNG, 'LineWidth', 1.2, 'Color', color.green, 'LineStyle','--')
plot(ENG_map_emissions.MGO.Load,ENG_map_emissions.MGO.s_CO_MGO, 'LineWidth', 1.2, 'Color', color.yellow)
xlabel('Load [-]')
ylabel('Specific emissions [g/kWhe]')
ax = gca;
ax.FontSize = Fontsize;
axis([0 1 0 1.5])
legend('SOx - MGO', 'SOx - LNG', 'PM - MGO', 'PM - LNG', 'CO - MGO','Location','northeast')
saveas(gcf,'EngineData/Maps/ENG_SOx_PM_CO.png')