close all;
%% Files and paths
addpath('EngineData/')
addpath('EngineData/Maps/')
addpath('SOFCData/')
addpath('Load_profile/')
addpath('Results/')

%% Simulation settings
% Design scenario
DS = 7 ;

% Ship
simulation.ships = ["CruiseBig" "CruiseMed" "CruiseFictive"];
simulation.ship = 3;

% Load profile
simulation.Load_Profiles = ["cruise", "transit", "year"];
simulation.Load_Profile = 1;

% Use commenting of inused component blocks to speed-up simulation (1 to activate) -> simulink must be open to use
% Don't forget to uncomment all blocks and save simulink before runnin at 0
simulation.commenting = 1;

% Dedicated simulation ('Initial', 'Adapted','Short', 'Longterm') -> when using full_run.m it should be on 'Initial'
simulation.run = 'Initial';

% Simulink version
simulation.current_simulink = 'Power_plant_v33.slx';
% Open simulink to use commenting
if simulation.commenting == 1
    open_system(simulation.current_simulink)
end

% Define fuel and engine type for dedicated design scenario
Design_scenario.ENG_type = [0 1 2 1 1 2 0]; % 0 is DG, 1 is DFG, 2 is GG
Design_scenario.Main_fuel = [0 1 1 1 1 1 1]; % 0 is MGO 1 is LNG

% Comment out components that are not necessary for certain DS (simulink file should be open to work)
switch DS
    case 1  % 1 = DS 1 (MGO gensets)
        DS_label = "1";
        if simulation.commenting == 1
            set_param([erase(simulation.current_simulink,'.slx'),'/SOFC'],'Commented','on');
            set_param([erase(simulation.current_simulink,'.slx'),'/ENG'],'Commented','off');
            set_param([erase(simulation.current_simulink,'.slx'),'/Prod'],'Commented','off');
        end
    case 2  % 2 = DS 2 (DFG genset)
        DS_label = "2";
        if simulation.commenting == 1
            set_param([erase(simulation.current_simulink,'.slx'),'/SOFC'],'Commented','on');
            set_param([erase(simulation.current_simulink,'.slx'),'/ENG'],'Commented','off');
            set_param([erase(simulation.current_simulink,'.slx'),'/Prod'],'Commented','off');        
        end    
    case 3  % 3 = DS 3 (Gas fuelled gensets)
        DS_label = "3";
        if simulation.commenting == 1
            set_param([erase(simulation.current_simulink,'.slx'),'/SOFC'],'Commented','on');
            set_param([erase(simulation.current_simulink,'.slx'),'/ENG'],'Commented','off');
            set_param([erase(simulation.current_simulink,'.slx'),'/Prod'],'Commented','off');            
        end
    case 4  % 4 = DS A (SOFC for Aux)
        DS_label = "A";
        if simulation.commenting == 1
            set_param([erase(simulation.current_simulink,'.slx'),'/SOFC'],'Commented','off');
            set_param([erase(simulation.current_simulink,'.slx'),'/ENG'],'Commented','off');
            set_param([erase(simulation.current_simulink,'.slx'),'/Prod'],'Commented','off');
        end
    case 5  % 5 = DS B (SOFC for Aux and Man)
        DS_label = "B";
        if simulation.commenting == 1
            set_param([erase(simulation.current_simulink,'.slx'),'/SOFC'],'Commented','off');
            set_param([erase(simulation.current_simulink,'.slx'),'/ENG'],'Commented','off');
            set_param([erase(simulation.current_simulink,'.slx'),'/Prod'],'Commented','off');
        end
    case 6  % 6 = DS C (SOFC for main operations)
        DS_label = "C";
        if simulation.commenting == 1
            set_param([erase(simulation.current_simulink,'.slx'),'/SOFC'],'Commented','off');
            set_param([erase(simulation.current_simulink,'.slx'),'/ENG'],'Commented','off');
            set_param([erase(simulation.current_simulink,'.slx'),'/Prod'],'Commented','off');
        end    
    case 7  % 7 = DS D (Only SOFC)
        DS_label = "D";
        if simulation.commenting == 1
            set_param([erase(simulation.current_simulink,'.slx'),'/SOFC'],'Commented','off');
            set_param([erase(simulation.current_simulink,'.slx'),'/ENG'],'Commented','on');
            set_param([erase(simulation.current_simulink,'.slx'),'/Prod'],'Commented','on');
        end
end

%% Load needed data
% Cruise ship profile (cruise, transit, or year)
load(append('Load_Profile_', simulation.ships(simulation.ship), '_' , simulation.Load_Profiles(simulation.Load_Profile),'.mat'));

% Rename loaded load profile to Load_profile
switch simulation.Load_Profile
    case 1
        Load_Profile = Load_Profile_cruise;
        simulation.a_time = 1;
    case 2
        Load_Profile = Load_Profile_transit;
        simulation.a_time = 1;
    case 3
        Load_Profile = Load_Profile_year;
        simulation.a_time = 5; % run for 5 years
end

% Runtime
time_step = 20*60; % [s]
t_stop = simulation.a_time * max(Load_Profile.Time); % [s] 
% Input from component sizing model (installed capacities in kW and kWh)
Installed = readtable(fullfile(append('output_components_',simulation.ships(simulation.ship),'.xlsx')),'UseExcel',true,'Sheet','Installed');  % usefullfile to reach file above current simulation
Installed.P_ENG_installed = max([Installed.P_DG_installed, Installed.P_DFG_installed, Installed.P_GG_installed], [], 2);

% Import component and emission data from component sizing model (use part of this data for componenent input
emission_data = readtable(fullfile('input.xlsx'),'UseExcel',true,'Sheet','Emissions'); % g/kwhe 
emission_data_WTT = readtable(fullfile('input.xlsx'),'UseExcel',true,'Sheet','Emissions_WTT'); % kg/kg to calculate from fuel consumption
component_data = readtable(fullfile('input.xlsx'),'UseExcel',true,'Sheet','Components','Range','A:F');
CO2_eq.CH4 = 25;
%% Emission factors to calculate carbon emissions directly from fuel % consumption
EF.MGO.CO2 = 0.875*44/12;                 % 0.875g/g in MGO molar mass CO2 is 44 molar mass C is 14
EF.LNG.CO2 = 0.75*44/12;                  % 0.875g/g in LNG
EF.LNG.CO2_ENG = (1-0.06) * 0.75*44/12;   % asssume average 6% methane slip in loss of C
%%
% Power conversion data
eta_ACDC = 0.96;        % used for SOFC in AC net
eta_DCDC = 0.98;        % Used for SOFC in DC net

% --- ENGINE ---
% Engine performance maps (data from Haseltaleb(2022) compensated for generator efficiency)
load("ENG_map.mat");
ENG_map_emissions.MGO = readtable('input_ENG.xlsx','Sheet','ENG_emissions_MGO');        
ENG_map_emissions.LNG = readtable('input_ENG.xlsx','Sheet','ENG_emissions_LNG');        
ENG_map_emissions.LNG2 = readtable('input_ENG.xlsx','Sheet','ENG_emissions_LNG_2');    

% Engine specifics
ENG.eta_el_LNG = 0.408;     % from wartsila 34DF (Ali) @@@ stil necessary?
ENG.eta_el_MGO = component_data.Value(strcmp(component_data.Component, 'DG') & strcmp(component_data.Fuel, 'MGO') & strcmp(component_data.Symbol, 'eta_el_avg'));         % from component sizing data
ENG.lifetime = 200000 * 3600;       % [s] hours converted to seconds
ENG.deg.Time = ENG.lifetime/200 * [0  19.9  20  39.9  40  59.9  60  79.9  80  99.9  100  119.9  120  139.9  140  159.9  160  179.9  180  199.9]; % From Cichiwicz (2015)
ENG.deg.sfc_increase =    [0    2  0.7  2.6  1.2  3.3  1.9  3.9  2.4   4.5  0.8   2.8   1.3    3.3  2.0  4.0    2.5    4.6  3.1    5.1];         % From Cichiwicz (2015)
ENG.a_trans_MGO = component_data.Value(strcmp(component_data.Component, 'DG') & strcmp(component_data.Fuel, 'MGO') & strcmp(component_data.Symbol, 'a_trans'));
ENG.a_trans_LNG = component_data.Value(strcmp(component_data.Component, 'GG') & strcmp(component_data.Fuel, 'LNG') & strcmp(component_data.Symbol, 'a_trans'));
ENG.a_pilot = component_data.Value(strcmp(component_data.Symbol, 'a_pilot')); % amount of pilot fuel in mass for DFG operation
 
ENG.limit = min(ENG_map.load); % Define minimim operating limit of single engine
ENG.P_limit = Installed.P_ENG_installed' .* [0 0 0 ENG.limit/3 ENG.limit/3 ENG.limit/2 0]; % To prevent switching on and off ENG above installed power of SOFC

% Fuel data
LHV.LNG = 50000;        % [kJ/kg]
LHV.MGO = 42800;        % [kJ/kg]

% --- SOFC ---
load("SOFC_map.mat");                                               % Map from Haseltaleb (2022)
load("SOFC_map_2D_R.mat");                                          % Map to include degradation in SOFC model
load("SOFC_map_compare.mat");                                       % Used to scale model from Map from Haseltaleb (2022) to model that includes lower fuel utilisation at lower load and variable heat loss
SOFC_map_sup = readtable('input_SOFC.xlsx','Sheet','SOFC_map');     % Supplementary SOFC map with heat efficiency and emissions
SOFC_map_deg = readtable('input_SOFC.xlsx','Sheet','degradation');  % SOFC map for steady state degradation rate for different load cases
SOFC.lifetime = 30000 * 60 * 60;                                        % [s] stack lifetime at 10%. net electric efficiency decrease
% retrieve full load efficiency at beginning of life and end of life from adapted performance map
SOFC.eta_el_bol = SOFC_map_compare.eta_adapted(end);  
SOFC.eta_el_eol = SOFC_map_compare.eta_adapted_eol(end);
SOFC.V_bol_full_load = interp2(SOFC_map_2D.I_norm, SOFC_map_2D.R_deg, SOFC_map_2D.V', 1, 0);
Z = griddata(SOFC_map_2D.I_norm, SOFC_map_2D.R_deg, SOFC_map_2D.P_norm', SOFC_map_2D.I_norm, 0.1);
SOFC.i_eol_full_load = interp1(Z,SOFC_map_2D.I_norm',1);
SOFC.V_eol_full_load = interp2(SOFC_map_2D.I_norm, SOFC_map_2D.R_deg, SOFC_map_2D.V', SOFC.i_eol_full_load, 0.1);
clear Z
% SOFC operation
% SOFC.max_current_ramp = 1 / (15*60);                          % during load changing (from Haseltaleb (2022), 15 minutes for 0-100% power)
% SOFC.max_current_ramp = 1 / (20*60*60);                       % including start-up phase of 20h
SOFC.max_current_ramp_up = 0.8* 1 / ((100/2)*60);               % 2%/min (from proof-of-concept experiment at DLR) 0.8 to convert from power to current
SOFC.max_current_ramp_down = 0.8 * -1 / ((100/8)*60);           % 8%/min (from proof-of-concept experiment at DLR) 0.8 to convert from power to current
% SOFC emission factors
SOFC.s_NOx = 0.001*(6.9+51.8)/2;                                % From measurements unreleased Gandiglio paper (load-independent)
SOFC.s_SOx = 0;
SOFC.s_CH4 = 0;
SOFC.s_CO = 0.001*(5.5+32.1)/2;                                 % From measurements unreleased Gandiglio paper (load-independent)
SOFC.s_PM = 0;                                          

% degradation parameters
SOFC.a_trans_threshold = 0.0005;               % treshold between steady and dynamic operation [%A/s]
SOFC.penalty_steady = 1;                       % no factor
SOFC.penalty_trans = 1.5;                      % 20% increased degradation in transient operation

% --- BAT ---
BAT.C_rate = component_data.Value(strcmp(component_data.Symbol, 'C_rate'));
BAT.SOC_LB = 0.2;
BAT.SOC_UB = 0.9;
BAT.charge_limits = [0.45 0.5 0.6 0.65];       % For double hysteresis control [SOC_L1, SOC_L2, SOC_U2, SOC_U1]
BAT.SOC_init = (BAT.charge_limits(2) + BAT.charge_limits(3))/2; % Initial and target SOC
BAT.eta_discharge = 0.95;
BAT.eta_charge = 0.9;
BAT.charge_rate = 0.5;                         % charge rate for SOC based control (slower recharching)
BAT.calender_aging.mean = 2.5*10^-5;           % [%/day] 
% Cycle data for NMC batteries from Preger (2020)
BAT.loadcycling.DOD = [0.2 0.6 1]; % depth of discharge
BAT.loadcycling.cycles = [2540 1634 429]; %

% --- BOIL ---
BOIL.eta_MGO = component_data.Value(strcmp(component_data.Component, 'BOIL') & strcmp(component_data.Fuel, 'MGO') & strcmp(component_data.Symbol, 'eta_th_avg'));
BOIL.eta_LNG = component_data.Value(strcmp(component_data.Component, 'BOIL') & strcmp(component_data.Fuel, 'LNG') & strcmp(component_data.Symbol, 'eta_th_avg'));
